'Option Strict On
Option Explicit On 

Imports System
Imports System.IO
Imports System.Xml
Imports System.Collections
Imports System.Drawing
Imports System.Drawing.Imaging
Imports System.Drawing.Printing
Imports System.Globalization
Imports System.Runtime.InteropServices
Imports System.Text
Imports Microsoft.ReportingServices.Interfaces

' Eine bermittlungserweiterung muss IDeliveryExtension und IExtension implementieren
Public Class WebgrafikUebermittlungsErweiterung
    Implements IExtension, IDeliveryExtension

    ' ---- Private Variablen 
    Private _bIsPrivilegedUser As Boolean = False
    ' Objekt, ber dass auf die fr die Erweiterung relevanten 
    ' Einstellungen des Berichtsservers zugriffen werden kann
    Private _reportServerInfo As IDeliveryReportServerInformation = Nothing
    ' Collection, in der die aus der RSReportServer.config-Datei ausgelesenen 
    ' Standardwerte fr die Benutzereinstellungen hinterlegt werden
    Private _colStandardwerte As New Collection
    ' Schlssel, mit denen die Benutzereinstellungen identifiziert werden
    Private Const _sIDGrafikHoeheCm As String = "GrafikHoeheCm"
    Private Const _sIDGrafikBreiteCm As String = "GrafikBreiteCm"
    Private Const _sIDAnzeigeText As String = "AnzeigeText"
    Private Const _sIDAusgabeDateiName As String = "AusgabeDateiName"
    Private Const _sIDAusgabeDateiFormat As String = "AusgabeDateiFormat"
    ' Text, mit dem Benutzer zur Eingabe der Benutzereinstellungen aufgefordert wird
    Private Const _sBeschriftungGrafikBreiteCm As String = "Breite der Grafik (in cm):"
    Private Const _sBeschriftungGrafikhoeheCm As String = "Hhe der Grafik (in cm):"
    Private Const _sBeschriftungAnzeigeText As String _
        = "Text, der in der Grafik angezeigt werden soll:"
    Private Const _sBeschriftungAusgabeDateiName As String _
        = "Name der Ausgabedatei (mit Pfad und Dateierweiterung):"
    Private Const _sBeschriftungAusgabeDateiFormat As String _
        = "Format der Ausgabedatei:"

    Public Sub New()
    End Sub 'New

    Public ReadOnly Property LocalizedName() As String _
            Implements IExtension.LocalizedName
        ' Gibt den Namen der Erweiterung in der Landessprache wieder.
        ' Idealerweise sollte Ihre Anwendung den Namen in der 
        ' eingestellten Landessprache zurckgeben
        Get
            Return "Webgrafik-Uebermittlungserweiterung-Beispiel"
        End Get
    End Property

    Public WriteOnly Property IsPrivilegedUser() As Boolean _
            Implements IDeliveryExtension.IsPrivilegedUser
        ' Erforderlich fr die Implementierung. 
        ' Wird von dieser Erweiterung nicht verwendet.
        Set(ByVal Value As Boolean)
            _bIsPrivilegedUser = Value
        End Set
    End Property
    Public WriteOnly Property ReportServerInformation() _
         As IDeliveryReportServerInformation _
             Implements IDeliveryExtension.ReportServerInformation
        ' ber diese Eigenschaft stellt der Berichtsserver 
        ' ein ReportServerInformation-Objekt bereit,
        ' das die Informationen enthlt, die fr eine bermittlungserweiterung 
        ' erforderlich bzw. hilfreich sind.
        ' In diesem Beispiel werden (in der Function bKannImageRendern) ber das hier 
        ' gelieferte Objekt festgestellt, ob die IMAGE-Rendering-Erweiterung
        ' installiert ist.
        Set(ByVal Value As IDeliveryReportServerInformation)
            _reportServerInfo = Value
        End Set
    End Property

    Private ReadOnly Property bKannImageRendern() As Boolean
        ' Hilfsfunktion, die ermittelt, ob IMAGE-Renderingerweiterung installiert ist.
        ' Wird nur von der Funktion renderBerichtAlsGrafik verwendet.
        Get
            ' Geht durch die installierten Renderingerweiterungen
            ' und gibt 'true' zurck, wenn IMAGE-Extension verfgbar ist
            Dim Ext As Microsoft.ReportingServices.Interfaces.Extension
            For Each Ext In _reportServerInfo.RenderingExtension
                If Ext.Name = "IMAGE" Then
                    Return True 'gefunden -> wir knnen IMAGE rendern
                    Exit For
                End If
            Next Ext
            ' Schleife ergebnislos durchlaufen -> IMAGE kann nicht gerendert werden
            Return False
        End Get
    End Property
    Public Sub SetConfiguration(ByVal configuration As String) _
            Implements IExtension.SetConfiguration
        ' Verarbeitung der XML-Daten, die in Berichtsserver-Konfigurationsdatei 
        ' (RSReportServer.config) gespeichert sind.
        ' configuration: XML-String aus der Berichtsserver-Konfigurationsdatei,
        '    der die Konfigurationsdaten unserer Erweiterung enthlt.
        Try
            ' XML-Dokument-Objekt erzeugen und Konfiguration hineinladen
            Dim xmlDoc As New XmlDocument
            xmlDoc.LoadXml(configuration)

            ' berprfen, ob wir wirklich beim richtigen Dokumentelement sind
            If xmlDoc.DocumentElement.Name = _
                    "WebgrafikUebermittlungsBeispiel-Konfiguration" Then
                ' -- alle Werte einlesen und in der _colStandardwerte-Collection merken
                Dim child As XmlNode
                For Each child In xmlDoc.DocumentElement.ChildNodes
                    Dim setting As setting
                    Select Case child.Name
                        ' erwartete Werte in Collection bernehmen
                    Case _sIDGrafikHoeheCm, _sIDGrafikBreiteCm, _sIDAnzeigeText _
                                , _sIDAusgabeDateiName, _sIDAusgabeDateiFormat
                            _colStandardwerte.Add(child.InnerText, child.Name)
                        Case Else ' bei unbekannten Werten mit Fehler abbrechen
                            Throw New exception(String.Format( _
                                "Fehler in WebgrafikUebermittlungsErweiterung." _
                                & "SetConfiguration: Einlesen Konfigurationsdaten " _
                                & "fehlgeschlagen: Unbekannte Konfigurations-" _
                                & "Einstellung '{0}'", child.Name))
                    End Select
                Next child
            End If

        Catch exception As exception
            Throw New exception("Fehler in WebgrafikUebermittlungsErweiterung." _
                & "SetConfiguration: Einlesen Konfigurationsdaten fehlgeschlagen:" _
                & exception.Message)
        End Try
    End Sub 'SetConfiguration
    Public ReadOnly Property ExtensionSettings() As Setting() _
        Implements IDeliveryExtension.ExtensionSettings
        ' Gibt Benutzereinstellungen fr die Webgrafikbermittlungserweiterung zurck.
        ' Sofern noch keine Benutzereinstellungen existieren, werden diese mit den 
        ' in _colStandardwerte abgelegten Standardwerten angelegt.
        Get
            Static settings As Setting() = Nothing
            ' -- falls settings-objekt nothing ist: settings m. Standardwerten erzeugen
            If settings Is Nothing Then
                ' Array fr Settings erzeugen
                settings = New Setting(4) {}
                ' -- je ein Setting fr jede der 5 benutzerdefinierte Einstellungen
                ' -- anlegen & fllen 
                settings(0) = neuesSetting(_sIDAusgabeDateiFormat _
                    , _sBeschriftungAusgabeDateiFormat)
                ' fr die Format-Einstellung ist nur "JPEG" und "TIFF" erlaubt
                settings(0).AddValidValue("JPEG", "JPEG")
                settings(0).AddValidValue("TIFF", "TIFF")
                settings(1) _
                    = neuesSetting(_sIDGrafikHoeheCm, _sBeschriftungGrafikhoeheCm)
                settings(2) _
                    = neuesSetting(_sIDGrafikBreiteCm, _sBeschriftungGrafikBreiteCm)
                settings(3) = neuesSetting(_sIDAnzeigeText, _sBeschriftungAnzeigeText)
                settings(4) = neuesSetting(_sIDAusgabeDateiName _
                    , _sBeschriftungAusgabeDateiName)
            End If
            ' in jedem Fall: aktuelle settings zurckgeben
            Return settings
        End Get
    End Property

    Private Function neuesSetting(ByVal sName As String _
            , ByVal sDisplayname As String) As Setting '
        ' Hilfsfunktion, die ein Setting erzeugt und 
        ' mit den bergebenen Werten befllt zurckgibt.
        ' Wird nur von Property ExtensionSettings verwendet.
        Dim sett As New Setting
        sett.Name = sName
        sett.DisplayName = sDisplayname
        sett.ReadOnly = False
        sett.Required = True
        sett.Value = _colStandardwerte(sName) 'sValue
        Return sett
    End Function
    Public Function ValidateUserData(ByVal settings() As Setting) As Setting() _
            Implements IDeliveryExtension.ValidateUserData
        ' Wird vom Berichtsserver verwendet, um festzustellen, ob ein bergebenes Set 
        ' von bermittlungserweiterungseinstellungen gltig ist. Listing 29.20
        ' settings: Ein Array von Setting[]-Objekten, das die vom Benutzer eingegebene 
        '     Erweiterungseinstellungen enthalten. 
        ' Rckgabe: Die berprften Einstellungen, wobei bei fehlerhaften Einstellungen 
        '     eine Meldung in der Error-Eigenschaft der betreffenden Einstellung 
        '     gesetzt sein muss, die der Client - z.B. der Berichts-Manager - anzeigt.
        Dim setting As setting
        For Each setting In settings
            Try
                Select Case setting.Name
                    Case _sIDGrafikHoeheCm, _sIDGrafikBreiteCm
                        If CDbl(setting.Value) < 0.1 Or CDbl(setting.Value) > 50 Then
                            setting.Error = "Wert muss zwischen 0.1 und 50.0 liegen"
                        End If
                    Case _sIDAnzeigeText
                        If Len(setting.Value) > 80 Then setting.Error _
                                                                = "Maximal 80 Zeichen"
                    Case _sIDAusgabeDateiName
                        ' falls gewnscht, berprfen Sie hier den AusgabeDateiNamen
                    Case _sIDAusgabeDateiFormat
                        If setting.Value <> "JPEG" And setting.Value <> "TIFF" Then
                            setting.Error = String.Format( _
                              "Tragen Sie hier 'JPEG' oder 'TIFF' ein.", setting.Value)
                        End If
                    Case Else
                        ' Unbekanntes Setting-Element -> Fehler ausgeben
                        setting.Error = String.Format("Unbekannte Einstellung '{0}'" _
                                                      , setting.Name)
                End Select
            Catch e As Exception
                ' Fehler beim bernehmen des Wertes -> neben Feld anzeigen
                setting.Error = "Fehler: " & e.Message
            End Try
        Next setting
        Return settings
    End Function 'ValidateUserData
    Public Function Deliver(ByVal notification As Notification) As Boolean Implements IDeliveryExtension.Deliver
        ' Kernfunktion der bermittlungserweiterung: Der Reportserver liefert 
        ' ein Notification-Objekt und erwartet dessen bermittlung.
        ' notification: Ein Notification-Objekt, das alle fr die bermittlung 
        '     erforderlichen Daten enthlt, insbesondere den Bericht betreffend 
        '     sowie die benutzerdefinierten Einstellungen, und Methoden fr
        '     Berichtsrendering und Statusrckmeldungen an den Berichtsserver umfasst.
        ' Rckgabe: Boolean-Wert, der Erfolg bzw. Misserfolg an Berichtsserver meldet.
        Dim bErfolg As Boolean = False

        ' Setzt den Status der Notification
        notification.Status = "Verarbeitung luft..."
        ' aktuellen Status dem Benutzer (typischerweise im Berichts-Manager) anzeigen 
        notification.Save()
        Try
            ' Bericht rendern, als Grafik speichern Erfolg durchreichen
            bErfolg = renderBerichtAlsGrafik(notification)
        Catch e As Exception
            ' Gib Fehler in Status-Eigenschaft des Notification-Objekts zurck
            notification.Status = "Fehler in WebgrafikUebermittlungsErweiterung." _
               & "Deliver: " + e.Message
            bErfolg = False
        Finally
            ' Berichtsserver soll es noch mal versuchen (bei Misserfolg)
            ' (Maximalanzahl Wiederholungsversuche wird fr Berichtsserver 
            '  global konfiguriert)
            notification.Retry = Not bErfolg
            ' Notification-Objekt in jedem Fall sichern
            notification.Save()
        End Try
        ' (Miss)Erfolg zurckgeben
        Return bErfolg
    End Function 'Deliver

    Private Function liesBenEinst(ByVal settings() As Setting, ByVal _sID As String) _
            As String
        ' Hilfsfunktion, mit der das Setting-Objekt nach einer Einstellung, 
        ' deren Schlssel im Parameter _sID bergeben durchsucht und wird.
        Dim setting As setting
        For Each setting In settings
            If setting.Name = _sID Then Return setting.Value
        Next
        ' Schleife wurde ergebnislos durchlaufen 
        ' -> gesuchte Einstellung existiert nicht -> Fehler auswerfen
        Throw New Exception(String.Format("Interner Fehler in Webgrafik" _
            & "UebermittlungsErweiterung.liesBenEinst:" _
            & "Kann Setting '{0}' nicht finden.", _sID))
    End Function
    Private Function renderBerichtAlsGrafik(ByRef notification As Notification) _
         As Boolean
        ' Hilfsfunktion, die die eigentliche Rendering-Arbeit bernimmt und nur von 
        ' Function Deliver aufgerufen wird.
        ' Rendert Bericht und speichert ihn (ggf. in mehreren) Grafik-Datei(en) ab.
        ' notification: vom Berichtsserver gelieferte notification-Objekt
        ' Rckgabe: Boolean-Wert, der ber Erfolg informiert 
        ' -- private Variablen
        ' vom Berichtsserver erzeugter (image)-stream mit gerendertem Bericht
        Dim renderedOutputStream As Stream
        ' Stringbuilder fr Logdatei
        Dim sbLogDatei As New StringBuilder
        ' boolean-variable, die ber Erfolg Auskunft gibt
        Dim bErfolg As Boolean = False

        Try
            ' Debug-Informationen ins eigene Logdatei schreiben
            sbLogDatei.Append(System.DateTime.Now.ToString() _
                + " Starte..." + Environment.NewLine)
            ' Sicherstellen, dass die bentigte Renderingerweiterung installiert ist 
            If Not bKannImageRendern Then Throw New exception( _
                "IMAGE-Renderingerweiterung ist nicht installiert.")
            Dim iAktSeite As Integer = 1
            ' aktueller Dateiname
            Dim sAktDateiName As String

            ' -- Bericht seitenweise rendern und abspeichern
            sbLogDatei.Append("Starte Bericht-Rendering... " + Environment.NewLine)
            Dim bLetzteSeite As Boolean = False
            While Not bLetzteSeite
                renderedOutputStream = renderEineSeite(notification, iAktSeite)
                ' -- sofern noch Daten gekommen sind: Grafik beschriften & speichern
                If renderedOutputStream.Length > 0 Then
                    ' Ausgabedateinamen ergnzen um lfd. (Seiten)Nr
                    sAktDateiName _
                         = liesBenEinst(notification.UserData, _sIDAusgabeDateiName)
                    ' falls keine Dateierweiterung mitgegeben: Dateiformat nehmen 
                    If InStr(sAktDateiName, ".") = 0 Then
                        sAktDateiName += "." _
                          + liesBenEinst(notification.UserData, _sIDAusgabeDateiFormat)
                    End If
                    sAktDateiName = Replace(sAktDateiName, "." _
                        , iAktSeite.ToString & ".", , 1)
                    beschrifteUndSpeicherGrafik(renderedOutputStream, sAktDateiName _
                         , liesBenEinst(notification.UserData, _sIDGrafikHoeheCm) _
                             / 2.54 - 1.0F _
                         , liesBenEinst(notification.UserData, _sIDAnzeigeText))


                    ' render-Informationen in unsere Logdatei schreiben
                    sbLogDatei.Append("'" & sAktDateiName _
                        & "' erfolgreich gespeichert." + Environment.NewLine)
                    ' nchster Renderdurchlauf auf Folgeseite 
                    iAktSeite += 1
                Else ' Lnge des Streams 0 -> keine Daten mehr zu rendern -> fertig!
                    bLetzteSeite = True
                End If
            End While

            ' Erfolgsmeldung fr die Anzeige im Berichtsmanager zurckgeben
            notification.Status = _
                String.Format("{0} Dateien in {1} im {2}-Format gespeichert.", _
                iAktSeite - 1, liesBenEinst(notification.UserData _
                    , _sIDAusgabeDateiName), _
                liesBenEinst(notification.UserData, _sIDAusgabeDateiFormat))
            bErfolg = True

        Catch exception As exception
            ' Fehlermeldung in Notification-Objekt (Anzeige im Berichts-Manager) 
            notification.Status = exception.Message
            ' ausfhrliche Fehlermeldung in unsere Logdatei schreiben
            sbLogDatei.Append(exception.Message + ": " + exception.StackTrace _
                + Environment.NewLine)
            bErfolg = False
        Finally
            ' in jedem Fall: unsere Logdatei speichern
            speicherLogDatei(sbLogDatei)
            sbLogDatei = Nothing
        End Try
        ' (Miss)Erfolg zurckgeben
        Return bErfolg
    End Function 'renderBerichtAlsGrafik

    Private Sub speicherLogDatei(ByVal sb As StringBuilder)
        ' Debug-Hilfsfunktion, die eine LogDatei ins Windows-system32-Verzeichnis schreibt
        Try
            Dim fs As New FileStream("WebgrafikUebermittlungsErweiterungLog.txt", FileMode.Append, FileAccess.Write)
            Dim writer As New StreamWriter(fs)
            writer.Write(sb.ToString())
            writer.Flush()
            writer.Close()
        Catch e As Exception
            Throw New IOException("Interner Fehler in WebgrafikUebermittlungsErweiterung.speicherLogDatei: " + e.Message)
        End Try
    End Sub 'speicherLogDatei

    Private Function renderEineSeite(ByRef notification As Notification, _
ByVal iAktSeite As Integer) As Stream

        ' XML-string mit Gerteinformation fr das Berichtsrendering 
        Dim sDeviceInfo As String
        ' der vom Berichtsserver zurckgegebene gerenderte Bericht
        Dim renderedOutputFile() As renderedOutputFile

        ' XML-string mit Gerteinformation fr das Berichtsrendering 
        ' zusammensetzen und dabei Ausgabeformat, Startseite, Seiten-
        ' hhe und -breite aus den vom Benutzereinstellungen bernehmen.
        sDeviceInfo = String.Format("<DeviceInfo><OutputFormat>{0}</OutputFormat>" _
             & "<StartPage>{1}</StartPage><PageHeight>{2}cm</PageHeight>" _
             & "<PageWidth>{3}cm</PageWidth></DeviceInfo>" _
             , liesBenEinst(notification.UserData, _sIDAusgabeDateiFormat) _
             , iAktSeite _
             , liesBenEinst(notification.UserData, _sIDGrafikHoeheCm) _
             , liesBenEinst(notification.UserData, _sIDGrafikBreiteCm))
        ' Bericht rendern
        renderedOutputFile = notification.Report.Render("IMAGE", sDeviceInfo)
        ' den Stream des ersten Files zurckgeben 
        ' (die IMAGE-renderingerweiterung erzeugt nur ein File pro Aufruf)
        Return renderedOutputFile(0).Data
    End Function
    Private Sub beschrifteUndSpeicherGrafik(ByRef renderedOutputStream As Stream _
        , ByVal sDateiName As String, ByVal dHoehe As Double _
        , ByVal sText As String)
        ' Image aus dem Berichts-Stream erzeugen
        Dim image As image = image.FromStream(renderedOutputStream)
        ' graphics-Objekt fr die weitere Bearbeitung erzeugen
        Dim graphics As graphics = graphics.FromImage(image)
        ' --- Beschriften der Grafik
        ' font- und brush-objekt erzeugen
        Dim font As New font("Arial", 16)
        Dim brush As New SolidBrush(Color.Black)
        ' Punkt fr die untere linke Ecke erzeugen (dabei in cm in inch umrechnen)
        Dim point As New PointF(96.0F, dHoehe * 96.0F)
        ' vom Benutzer vorgergebenen Text in Grafik zeichnen
        graphics.DrawString(sText, font, brush, point)
        ' Grafik in Ausgabedatei speichern
        image.Save(sDateiName)
        graphics.Dispose()
        image.Dispose()
    End Sub
End Class





