Option Strict On
Option Explicit On 

Imports System
Imports System.ComponentModel
Imports Microsoft.ReportingServices.DataProcessing

Public Class XmlCommand
    Implements IDbCommand
    ' die Verbindung als XMLConnection-Objekt
    Private _connection As XmlConnection
    ' der vom client gelieferte Kommandotext
    Private _sCmdText As String
    ' Parameter-Collection des Kommandos
    ' hat in diesem Beispiel nur Platzhalterfunktion
    Private _parameters As New XmlDataParameterCollection

    Public Sub New(ByVal connection As XmlConnection)
        _connection = connection
    End Sub 'New
    '---- Implementierung der erforderlichen Eigenschaften

    Public Property CommandText() As String Implements IDbCommand.CommandText
        ' -- verwaltet den Kommandotext in privater Variablen
        Get
            Return _sCmdText
        End Get
        Set(ByVal Value As String)
            _sCmdText = Value
        End Set
    End Property

    Public Property CommandTimeout() As Integer Implements IDbCommand.CommandTimeout
        ' -- Gibt Kommando-Timeout zurck
        ' -- 0 steht fr eine unendlichen Timeout-Zeit
        Get
            Return 0
        End Get
        Set(ByVal Value As Integer)
        End Set
    End Property

    Public Property CommandType() As CommandType Implements IDbCommand.CommandType
        ' - gibt den Typ des Kommandos zurck
        ' - In diesem Beispiel wird nur CommandType.Text untersttzt.
        Get
            Return CommandType.Text
        End Get
        Set(ByVal Value As CommandType)
            If Value <> CommandType.Text Then
                Throw New NotSupportedException
            End If
        End Set
    End Property

    Public ReadOnly Property XmlParameters() As XmlDataParameterCollection
        ' -- (xxxx kann die weg?) Gibt die XmlParameter-Collection zurck
        ' - In diesem Beispiel haben die Parameter nur Platzhalterfunktion .
        Get
            Return _parameters
        End Get
    End Property

    Public ReadOnly Property Parameters() As IDataParameterCollection _
            Implements IDbCommand.Parameters
        ' -- Gibt die XmlParameter-Collection zurck
        ' - In diesem Beispiel haben die Parameter nur Platzhalterfunktion .
        Get
            Return _parameters
        End Get
    End Property


    Public Property Transaction() As IDbTransaction Implements IDbCommand.Transaction
        '- In diesem Beispiel werden keine Transaktionen untersttzt.	
        Get
            Throw New NotSupportedException
        End Get
        Set(ByVal Value As IDbTransaction)
            Throw New NotSupportedException
        End Set
    End Property
    '---- Implementierung der erforderlichen Methoden
    Public Sub Cancel() Implements IDbCommand.Cancel
        '-- In diesem Beispiel wird das Abbrechen nicht untersttzt.
        Throw New NotSupportedException
    End Sub 'Cancel

    Public Function CreateParameter() As IDataParameter _
            Implements IDbCommand.CreateParameter
        Return CType(New XmlDataParameter, IDataParameter)
    End Function 'CreateParameter

    Public Overloads Function ExecuteReader(ByVal behavior As CommandBehavior) _
             As IDataReader Implements IDbCommand.ExecuteReader
        '-- holt die Daten aus der Datenquelle und gibt ein DataReader-Objekt zurck,
        '-- ber das der Benutzer die Ergebnisse abfragen kann. 
        '- In diesem Beispiel ist die CommandBehavior-Einstellung wirkungslos.
        ' berprfen, ob es eine gltige und offene Verbindung gibt 
        If _connection Is Nothing _
                OrElse _connection.State <> System.Data.ConnectionState.Open Then
            Throw New InvalidOperationException("Verbindung muss gltig & offen sein!")
        End If
        ' Kommando ausfhren	
        Dim reader As New XmlDataReader(_sCmdText)
        reader.liesXmlDataDocument(_sCmdText)
        Return reader
    End Function 'ExecuteReader

    Public Sub Dispose() Implements IDbCommand.Dispose
    End Sub 'Dispose
End Class 'XmlCommand
